@@ -6,7 +6,7 @@ const colors = {
 }
 
 const notification_template = info => `
-	<div class="notification" style="background-color:${colors[type]};">
+	<div class="notification" style="background-color:${colors[info.notification_type]};">
 		<b>${info.title}</b><br/>
 		<span>${info.body}</span>
 
@@ -20,11 +20,11 @@ function htmlToElement(html) {
    	return template.content.firstChild;
 }
 
-function notify(title,msg,type='info',reload=false) {
+function notify(title,msg,notification_type='info',reload=false) {
 	info = {
 		'title':title,
 		'body':msg,
-		'type':type
+		'notification_type':notification_type
 	}
 
 	var element = htmlToElement(notification_template(info));
@@ -33,3 +33,22 @@ function notify(title,msg,type='info',reload=false) {
 
 	setTimeout(function(e){e.remove();},7000,element);
 }
+
+function notifyCallback(request) {
+	var body = request.response;
+	var status = request.status;
+
+	if (status == 200) {
+		var notification_type = 'info';
+		var title = "Success!";
+		var msg = body.desc || body;
+	}
+	else {
+		var notification_type = 'warning';
+		var title = "Error: " + body.error.type;
+		var msg = body.error.desc || "";
+	}
+
+
+	notify(title,msg,notification_type);
+}
